package org.supermind.crawl;

import org.apache.nutch.protocol.ProtocolOutput;
import org.supermind.crawl.util.ChainedScatterTable;
import org.supermind.crawl.util.ScatterPurger;

import java.io.IOException;
import java.net.URL;
import java.util.zip.Checksum;

/**
 * Caches URL checksums via a chained scatter table. When there are too many
 * urls, evicted checksums are persisted.
 */
public class CachingFetchedURLs implements FetchedURLs {
  private ChainedScatterTable cache;
  private ScatterPurger purger;
  private Checksum checksum;

  public CachingFetchedURLs() {
  }

  public void init() throws IOException {

  }

  public synchronized boolean contains(URL url) {
    long chksum = getChecksum(url);
    boolean inCache = cache.contains(chksum);
    if (!inCache) {
      try {
        return purger.contains(chksum);
      } catch (IOException e) {
        e.printStackTrace();
        return false;
      }
    } else {
      return true;
    }
  }

  public synchronized void insert(ScheduledURL url, ProtocolOutput output) {
    long chksm = getChecksum(url.url);
    cache.insert(chksm);
  }

  public ScheduledURL get(long id) {
    throw new UnsupportedOperationException("get() not supported");
  }
  
  public void close() throws IOException {
    purger.close();
  }

  public void setPurger(ScatterPurger purger) {
    this.purger = purger;
    cache = new ChainedScatterTable(purger);
  }

  public void setChecksum(Checksum checksum) {
    this.checksum = checksum;
  }

  /**
   * Create a 64-bit checksum by merging a 32-bit host checksum with the url's
   * 32-bit checksum. By using host checksum as sig. bits, urls can be easily
   * sorted by host.
   *
   * @param url
   * @return
   */
  protected long getChecksum(URL url) {
    String host = url.getHost();
    byte[] bytes = host.getBytes();
    checksum.update(bytes, 0, bytes.length);
    long hostchecksum = checksum.getValue() << 32;
    checksum.reset();

    bytes = url.toString().getBytes();
    checksum.update(bytes, 0, bytes.length);
    long urlchecksum = checksum.getValue();
    checksum.reset();

    return hostchecksum | urlchecksum;
  }
}
