package org.supermind.crawl;

import java.util.Iterator;
import java.io.IOException;
import java.net.URL;

/**
 * Source of URLs to seed a crawl. Possible implementations include a file-based
 * source and database-backed source.
 * <p>
 * SeedURLs may optionally have an id. This allows FetchList filters to
 * constrain a crawl to its seed's host/domain.
 * </p>
 */
public interface CrawlSeedSource {
  /**
   * Get iterator of seed URLs.
   * @return
   * @throws IOException
   */
  Iterator<SeedURL> getSeedURLs() throws IOException;

  /**
   * Get seed URL corresponding to an index. (optional operation)
   * @param index
   * @return SeedURL
   * @throws UnsupportedOperationException if CrawlSeedSource doesn't support random access
   */
  SeedURL getSeedURL(int index);

  /**
   * Close resources.
   * @throws IOException
   */
  void close() throws IOException;
}


