package org.supermind.crawl;

import org.apache.nutch.db.Page;
import org.apache.nutch.fs.NutchFileSystem;
import org.apache.nutch.io.ArrayFile;
import org.apache.nutch.pagedb.FetchListEntry;
import org.apache.nutch.parse.Parse;
import org.apache.nutch.parse.ParseData;
import org.apache.nutch.parse.ParseText;
import org.apache.nutch.protocol.Content;
import org.apache.nutch.util.LogFormatter;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * Persists a fetched page to a Nutch segment.
 */
public class NutchSegmentPostFetchProcessor implements PostFetchProcessor {
  static Logger LOG =
      LogFormatter.getLogger(NutchSegmentPostFetchProcessor.class.getName());

  private ArrayFile.Writer fetcherWriter;         // the output
  private ArrayFile.Writer contentWriter;
  private ArrayFile.Writer parseTextWriter;
  private ArrayFile.Writer parseDataWriter;
  private NutchFileSystem nfs;
  private String directory;
  private boolean overwrite;

  public void process(FetcherOutput fo, Content content,
                         Parse parse) throws IOException {
    boolean parsing = (parse != null);
    if (fetcherWriter == null) init(parsing);
    try {
      synchronized (fetcherWriter) {

        FetchListEntry fle = new FetchListEntry( // create the necessary nutch objects
            false, new Page(fo.getUrl(), fo.getMD5Hash()), new String[0]);
        org.apache.nutch.fetcher.FetcherOutput nutchfo = new
            org.apache.nutch.fetcher.FetcherOutput(fle, fo.getMD5Hash(), fo.getProtocolStatus());

        fetcherWriter.append(nutchfo); // append
        contentWriter.append(content);
        if (parsing) {
          parseTextWriter.append(new ParseText(parse.getText()));
          parseDataWriter.append(parse.getData());
        }
      }
    } catch (Throwable t) {
      LOG.severe("error writing output:" + t.toString());
      t.printStackTrace();
    }
  }

  public void close() throws IOException {
    if(fetcherWriter != null) fetcherWriter.close();              // close databases
    if(contentWriter != null) contentWriter.close();
    if (parseTextWriter != null) {
      parseTextWriter.close();
      parseDataWriter.close();
    }
    LOG.info("All NutchSegmentPostFetchProcessor resources closed.");
  }

  private void init(boolean parsing) throws IOException {
    LOG.info("Initializing NutchSegmentPostFetchProcessor");
    if (overwrite) {
      File dir = new File(directory);
      nfs.delete(dir);
    }

    // Set up in/out streams
    if (parsing) {
      fetcherWriter = new ArrayFile.Writer
          (nfs, new File(directory, FetcherOutput.DIR_NAME).toString(),
              org.apache.nutch.fetcher.FetcherOutput.class);
    } else {
      fetcherWriter = new ArrayFile.Writer
          (nfs, new File(directory, FetcherOutput.DIR_NAME_NP).toString(),
              org.apache.nutch.fetcher.FetcherOutput.class);
    }
    contentWriter = new ArrayFile.Writer
        (nfs, new File(directory, Content.DIR_NAME).toString(), Content.class);
    if (parsing) {
      parseTextWriter = new ArrayFile.Writer(nfs,
          new File(directory, ParseText.DIR_NAME).toString(), ParseText.class);
      parseDataWriter = new ArrayFile.Writer(nfs,
          new File(directory, ParseData.DIR_NAME).toString(), ParseData.class);
    }
  }

  public void setNfs(NutchFileSystem nfs) {
    this.nfs = nfs;
  }

  public void setDirectory(String directory) {
    this.directory = directory;
  }

  public void setOverwrite(boolean overwrite) {
    this.overwrite = overwrite;
  }
}
