package org.supermind.crawl.scope;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Limits the scope of an operation through {@link ScopeFilter}s. ScopeFilters
 * are run sequentially, in the same order that they are added.
 */
public abstract class AbstractScope<T> {
  protected List<ScopeFilter> filters = new LinkedList();

  /**
   * Should {@link #isInScope} return true when all filters abstain?  
   */
  protected boolean allowByDefault = false;

  /**
   * Add a ScopeFilter.
   * @param filter
   */
  public synchronized void add(ScopeFilter<T> filter) {
    filters.add(filter);
  }

  /**
   * Is the parameter within this scope? If all filters abstain,
   * {@link #allowByDefault} is returned.
   * @param t input
   * @return
   */
  public boolean isInScope(T t) {
    boolean hasAllow = false;
    for (Iterator<ScopeFilter> it = filters.iterator(); it.hasNext();) {
      ScopeFilter f = it.next();
      final int result = f.filter(t);
      if (result == ScopeFilter.REJECT)
        return false;
      else if (result == ScopeFilter.ALLOW) hasAllow = true;
    }
    if (hasAllow) return true;
    return allowByDefault;
  }

  public void setAllowByDefault(boolean allowByDefault) {
    this.allowByDefault = allowByDefault;
  }

  public void setFilters(List<ScopeFilter> filters) {
    this.filters = filters;
  }
}
