package org.supermind.crawl.scope;

import org.apache.nutch.parse.Parse;
import org.apache.nutch.protocol.Content;
import org.supermind.crawl.FetcherOutput;
import org.supermind.crawl.util.MD5Persister;
import org.supermind.crawl.PostFetchProcessor;
import org.supermind.crawl.util.MD5Persister;

import java.io.IOException;

/**
 * Writes MD5s to a MapFile for easy comparison. Similar in concept to
 * how FetchedURLs works.
 */
public class MapFileContentSeenFilter
    implements ScopeFilter<PostFetchScope.Input>, PostFetchProcessor {
  private MD5Persister persister;

  public MapFileContentSeenFilter() {
  }

  public synchronized void process(FetcherOutput fo, Content content, Parse parse) throws IOException {
    this.persister.add(fo.getMD5Hash());
  }

  public void close() throws IOException {
    if(persister != null) persister.close();
  }

  /**
   * Check webdb to see if the page's MD5 exists.
   *
   * @param o
   * @return
   */
  public int filter(PostFetchScope.Input input) {
    try {
      return persister.contains(input.fo.getMD5Hash()) ? REJECT : ALLOW;
    } catch (IOException e) {
      e.printStackTrace();
      return ABSTAIN;
    }
  }

  public void setPersister(MD5Persister persister) throws IOException {
    this.persister = persister;
    this.persister.init();
  }
}
