package org.supermind.crawl.scope;

import org.supermind.crawl.CrawlSeedSource;

import java.net.URL;

/**
 * Allows a URL if its parent has the same host as its seed. This effectively
 * allows all URLs within the same host, as well as one link outside the seed's 
 * host.
 */
public class OneExternalLinkFLFilter implements ScopeFilter<FetchListScope.Input>{
  private CrawlSeedSource seedSource;
  public int filter(FetchListScope.Input input) {
    URL parent = input.parent.url;
    URL seed = seedSource.getSeedURL(input.parent.seedIndex).getUrl();
    if(parent.getHost().equals(seed.getHost())) return ALLOW;

    return REJECT;
  }

  /**
   * Set {@link CrawlSeedSource}. Note: CrawlSeedSource implementation
   * <b>must</b> support random access by seed id.
   * @param seedSource
   * @see CrawlSeedSource#getSeedURL(int)
   */
  public void setSeedSource(CrawlSeedSource seedSource) {
    this.seedSource = seedSource;
  }
}
