package org.supermind.crawl.scope;

import org.apache.nutch.db.WebDBReader;
import org.apache.nutch.fs.NutchFileSystem;
import org.supermind.crawl.FetcherOutput;

import java.io.File;
import java.io.IOException;

/**
 * Uses Nutch's WebDB and a page's Md5 hash to determine if a page has been seen.
 */
public class WebDBContentSeenFilter implements ScopeFilter<FetcherOutput> {
  private WebDBReader reader;
  private NutchFileSystem nfs;
  private String directory;

  public WebDBContentSeenFilter(NutchFileSystem nfs, String directory) {
    this.nfs = nfs;
    this.directory = directory;
  }

  public int filter(FetcherOutput fetcherOutput) {
    try {
      if(reader == null) init();
      boolean exist = reader.pageExists(fetcherOutput.getMD5Hash());
      return exist ? REJECT : ALLOW;
    } catch (IOException e) {
      e.printStackTrace();
      return ABSTAIN;
    }
  }

  private void init() throws IOException {
    reader = new WebDBReader(nfs, new File(directory));
  }
}
