package org.supermind.crawl;

import junit.framework.TestCase;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Logger;
import java.util.logging.Level;

public class TestDefaultFetchList extends TestCase {
  DefaultFetchList fl = new DefaultFetchList();
  ScheduledURL parent = new ScheduledURL(-1, -1, null);

  public TestDefaultFetchList() {
    Logger.getLogger(
        TestDefaultFetchList.class.getPackage().getName()).setLevel(Level.FINEST);
  }

  protected void setUp() throws Exception {
    for (int i = 0; i < 20; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }
    for (int i = 0; i < 10; ++i) {
      fl.queue(parent, new URL("http://www.apache.org/" + i + ".html"));
    }
  }

  public void testQueue() throws MalformedURLException {
    assertEquals("www.nutch.org", fl.next().getName());
    assertEquals("www.apache.org", fl.next().getName());
    fl.close();

    for (int i = 0; i < 5; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }
    for (int i = 0; i < 10; ++i) {
      fl.queue(parent, new URL("http://www.apache.org/" + i + ".html"));
    }
    for (int i = 0; i < 15; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }

    assertEquals("www.nutch.org", fl.next().getName());
    assertEquals("www.apache.org", fl.next().getName());
    fl.close();


    for (int i = 0; i < 20; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }
    for (int i = 0; i < 18; ++i) {
      fl.queue(parent, new URL("http://www.apache.org/" + i + ".html"));
    }

    HostQueue nutchHost = fl.next();
    assertEquals("www.nutch.org", nutchHost.getName());
    ScheduledURL[] popped = nutchHost.pop();

    for (int i = 0; i < 29; ++i) {
      fl.queue(parent, new URL("http://www.supermind.org/" + i + ".html"));
    }
    fl.release(nutchHost, popped.length, 0); // assume more than 2 URLs popped

    assertEquals("www.supermind.org", fl.next().getName());
    assertEquals("www.apache.org", fl.next().getName());
    assertEquals("www.nutch.org", fl.next().getName());
    fl.close();

    // test wait time
    fl.setWaitFactor(1);
    for (int i = 0; i < 200; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }
    for (int i = 0; i < 10; ++i) {
      fl.queue(parent, new URL("http://www.apache.org/" + i + ".html"));
    }
    for (int i = 0; i < 5; ++i) {
      fl.queue(parent, new URL("http://www.supermind.org/" + i + ".html"));
    }

    nutchHost = fl.next();
    assertEquals("www.nutch.org", nutchHost.getName());
    fl.release(nutchHost, popped.length, Integer.MAX_VALUE);
    assertEquals("www.apache.org", fl.next().getName());
    assertEquals("www.supermind.org", fl.next().getName());
  }

  public void testNext() throws MalformedURLException {
    HostQueue nutchHost = fl.next();
    assertEquals("www.nutch.org", nutchHost.getName());
    ScheduledURL[] popped = nutchHost.pop();
    fl.release(nutchHost, popped.length, 0);

    assertEquals("www.apache.org", fl.next().getName());
    fl.close();


    for (int i = 0; i < 50; ++i) {
      fl.queue(parent, new URL("http://www.nutch.org/" + i + ".html"));
    }
    for (int i = 0; i < 10; ++i) {
      fl.queue(parent, new URL("http://www.apache.org/" + i + ".html"));
    }

    nutchHost = fl.next();
    assertEquals("www.nutch.org", nutchHost.getName());
    popped = nutchHost.pop();
    fl.release(nutchHost, popped.length, 0);

    assertEquals("www.nutch.org", fl.next().getName());
    fl.close();
  }

  public void testContains() throws MalformedURLException {
    assertTrue(fl.contains(new URL("http://www.nutch.org/1.html")));
    assertTrue(fl.contains(new URL("http://www.apache.org/1.html")));

    HostQueue nutchHost = fl.next();
    assertEquals("www.nutch.org", nutchHost.getName());
    ScheduledURL[] popped = nutchHost.pop();

    assertFalse(fl.contains(new URL("http://www.nutch.org/1.html")));
    assertTrue(fl.contains(new URL("http://www.nutch.org/19.html")));
    assertTrue(fl.contains(new URL("http://www.apache.org/1.html")));

    fl.release(nutchHost, popped.length, 0);
    assertFalse(fl.contains(new URL("http://www.nutch.org/1.html")));
    assertTrue(fl.contains(new URL("http://www.nutch.org/19.html")));
    assertTrue(fl.contains(new URL("http://www.apache.org/1.html")));
  }

}
